import json
import re
import random

from main_clear.sci_clear import get_maplef_items
from config.config import log
# 通用公有变量
public_id = 0
# 数据预处理
class DataPreProcessing():
    def __init__(self, logger=None):
        self.logger = logger
        self.log_msg = "id : {id} -> {type} -> {message}"
    # 清洗函数
    def clear_func(self, content):
        if content in {'', None}:
            return ''
        # 将content字符串化,防止content是int/float型
        if isinstance(content, str) is False:
            if isinstance(content, int) or isinstance(content, float):
                return str(content)
        try:
            # 进行文本清洗
            content_clear = get_maplef_items(content)
        except Exception as e:
            # 通用公有变量
            global public_id
            # 日志采集
            print(self.log_msg.format(id=public_id,
                                      type="清洗错误: " + str(e),
                                      message=str(content))) if self.logger is None else None
            self.logger.error(self.log_msg.format(id=public_id,
                                                  type="清洗错误: " + str(e),
                                                  message=str(content))) if self.logger is not None else None
            # 对于无法清洗的文本通过正则表达式直接获取文本中的中文字符
            content_clear = re.sub(r'[^\u4e00-\u9fa5]', '', content)
            raise e

        return content_clear

    # 重叠截取长文本进行Sentence-Bert训练
    def truncate_func(self, content):
        # 设置长文本截断长度
        cut_length = 150
        # 设置截断重叠长度
        overlap = 10
        content_cut_list = []

        # 若文本长度小于等于截断长度,则取消截取直接返回
        cont_length = len(content)
        if cont_length <= cut_length:
            content_cut_list = [content]
            return content_cut_list

        # 若文本长度大于截断长度,则进行重叠截断
        # 设定文本截断尾部合并阈值(针对尾部文本根据长度进行合并)
        # 防止截断后出现极短文本影响模型效果
        tail_merge_value = 0.5 * cut_length
        for i in range(0, cont_length, cut_length - overlap):
            tail_idx = i + cut_length
            cut_content = content[i:tail_idx]
            # 保留单词完整性
            # 判断尾部字符
            if cont_length - tail_idx > tail_merge_value:
                for j in range(len(cut_content) - 1, -1, -1):
                    # 判断当前字符是否为字母或者数字
                    # 若不是字母或者数字则截取成功
                    if re.search('[A-Za-z]', cut_content[j]) is None:
                        cut_content = cut_content[:j + 1]
                        break
            else:
                cut_content = content[i:]
            # 判断头部字符
            if i != 0:
                for k in range(len(cut_content)):
                    # 判断当前字符是否为字母或者数字
                    # 若不是字母或者数字则截取成功
                    if re.search('[A-Za-z]', cut_content[k]) is None:
                        cut_content = cut_content[k + 1:]
                        break
            # 将头部和尾部都处理好的截断文本存入content_cut_list
            content_cut_list.append(cut_content)
            # 针对尾部文本截断长度为140-150以及满足尾部合并阈值的文本
            # 进行重叠截断进行特殊处理
            if cont_length - tail_idx <= tail_merge_value:
                break

        return content_cut_list

    # 全文本数据清洗
    def content_clear_func(self, content):
        # 文本清洗
        content_clear = self.clear_func(content)
        # 去除文本中的空格以及空字符串
        content_clear = re.sub(r',+', ',', re.sub(r'[\s_]', '', content_clear))

        # 去除题目开头"【题文】(多选)/(..分)"
        content_clear = re.sub(r'\[题文\]', '', content_clear)
        content_clear = re.sub(r'(\([单多]选\)|\[[单多]选\])', '', content_clear)
        content_clear = re.sub(r'(\(\d{1,2}分\)|\[\d{1,2}分\])', '', content_clear)
        # # 将文本中的选项"A.B.C.D."改为";"
        # content_clear = re.sub(r'[ABCD]\.', ';', content_clear)
        # # 若选项中只有1,2,3或(1)(2)(3),或者只有标点符号,则过滤该选项
        # content_clear = re.sub(r'(\(\d\)[、,;\.]?)+\(\d\)|\d[、,;]+\d', '', content_clear)
        # 去除题目开头(...年...[中模月]考)文本
        head_search = re.search(r'^(\(.*?[\)\]]?\)|\[.*?[\)\]]?\])', content_clear)
        if head_search is not None and 5 < head_search.span(0)[1] < 40:
            head_value = content_clear[head_search.span(0)[0] + 1:head_search.span(0)[1] - 1]
            if re.search(r'.*?(\d{2}|[模检测训练考试验期省市县外第初高中学]).*?[模检测训练考试验期省市县外第初高中学].*?', head_value):
                content_clear = content_clear[head_search.span(0)[1]:].lstrip()
        # 对于只有图片格式以及标点符号的信息进行特殊处理(去除标点符号/空格/连接符)
        if re.sub(r'[\.、。,;\:\?!#\-> ]+', '', content_clear) == '':
            content_clear = ''

        return content_clear

    # 数据清洗与长文本重叠截取处理
    def content_clear_process(self, data):
        # 初始化content_clear
        content_clear = ''
        # 全文本数据清洗
        content_clear = self.content_clear_func(data)

        # 重叠截取长文本用于进行Sentence-Bert训练
        # content_cut_list = self.truncate_func(content_clear)

        return content_clear

from config.config import log
dpp = DataPreProcessing(logger=log)
def convert_option(options):
    option_desc = ""
    for i, option in enumerate(options):
        if i == 0:
            option_desc += "A. " + dpp.content_clear_func(options[i]["content"])
        if i == 1:
            option_desc += "\n" + "B. " + dpp.content_clear_func(options[i]["content"])
        if i == 2:
            option_desc += "\n" + "C. " + dpp.content_clear_func(options[i]["content"])
        if i == 3:
            option_desc += "\n" + "D. " + dpp.content_clear_func(options[i]["content"])
    return option_desc

def build_params(request_dict):

    content = dpp.content_clear_func(request_dict["topic_text"])
    option = request_dict["option"]
    parse = dpp.content_clear_func(request_dict["parse"]) if len(request_dict["parse"]) != 0 else request_dict["parse"]
    if len(option) != 0:
        option = convert_option(option)
    if option:
        content = content + "\n" + option
    content = content + "\n" + "解析过程\n"
    content += parse
    sentence = "根据题目描述和解析过程\n'''题目描述:\n" + content + "'''\n，给出考察的知识点。"
    log.debug("清洗后的句子"+sentence)
    return sentence